﻿<#
.SYNOPSIS
    A script that demonstrates PowerShell integration with REST API.
.DESCRIPTION
    Authenticate and call "Get or delete certificate" REST API.
.PARAMETER uemHostAndPort
    Host and port of the UEM.
.PARAMETER tenantGuid
    TenantGuid to use for authentication.
.PARAMETER authType
    Authentication type to use.  (Local or AD).
.PARAMETER username
    Username to use for authentication.
.PARAMETER password
    Password to use for authentication.
.PARAMETER domain
    Domain to use for authentication if authType=AD.
.PARAMETER action
    Action to perform. (get certificate or delete certificate)
.PARAMETER certificateGuid
    Certificate guid by which the search or deleting will be performed.
.EXAMPLE
    get certificate
        .\GetDeletePKICertificate.ps1 -uemHostAndPort server01:18084 -tenantGuid SRP00000 -authType Local -username admin -password password -domain example -action get -certificateGuid aff621f9-8507-42c9-ad40-7ab143f5585f
    delete certificate
        .\GetDeletePKICertificate.ps1 -uemHostAndPort server01:18084 -tenantGuid SRP00000 -authType Local -username admin -password password -domain example -action delete -certificateGuid aff621f9-8507-42c9-ad40-7ab143f5585f
#>
Param(
    [Parameter(
        Mandatory=$true,
        HelpMessage="UEM Host and port."
    )]
    [ValidateNotNullorEmpty()]
    [string]$uemHostAndPort,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Tenant guid to authenticate with."
    )]
    [ValidateNotNullorEmpty()]
    [string]$tenantGuid,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Choose (AD | Local) authType."
    )]
    [ValidateSet('AD', 'Local')]
    [string]$authType,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Username to authenticate with."
    )]
    [ValidateNotNullorEmpty()]
    [string]$username,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Password to authenticate with."
    )]
    [ValidateNotNullorEmpty()]
    [string]$password,

    [Parameter(
        Mandatory=$false,
        HelpMessage="Domain for AD authenticate type."
    )]
    [string]$domain,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Choose (get | delete) action."
    )]
    [ValidateSet('get', 'delete')]
    [string]$action,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Certificate guid to get specific one."
    )]
    [ValidateNotNullorEmpty()]
    [string]$certificateGuid
)

$Global:authString = $null;
$Global:uemBaseUrl = New-Object -TypeName System.Text.StringBuilder;
$Global:isGet = $action -eq "get";

[System.Net.ServicePointManager]::SecurityProtocol = [System.Net.SecurityProtocolType]::Tls12;

<#
    * Generate authorization header required to perform the REST call.
    *
    * @return
    *       True if successful, false otherwise.
#>
function setup()
{
    $methodName = "setup()";
    enterMethod($methodName);
    $provider = $null;

    $Global:uemBaseUrl.Append("https://");
    $Global:uemBaseUrl.Append($uemHostAndPort);
    $Global:uemBaseUrl.Append("/");
    $Global:uemBaseUrl.Append($tenantGuid);
    $Global:uemBaseUrl.Append("/api/v1");

    # Create and set the headers for the REST request
    $headers = @{"Content-Type" = "application/vnd.blackberry.authorizationrequest-v1+json"};

    # Create body for REST call
    $body = @{};
    $body.add("provider",$authType);
    $body.add("username", $username);
    $body.add("password", [Convert]::ToBase64String([System.Text.Encoding]::UTF8.GetBytes($password)));
    if($authType -eq "AD")
    {
        if(-not($domain))
        {
            throw [System.ArgumentNullException]"The domain argument is required when authType=AD";
        }
        $body.add("domain", $domain);
    }

    $request = New-Object -TypeName System.Text.StringBuilder;
    $request.Append($Global:uemBaseUrl);
    $request.Append("/util/authorization");

    # Invoke RESTMethod to call authorization route to generate the authorization headers
    try
    {
        $response = Invoke-RestMethod -Method POST -Headers $headers -Uri $request.ToString() -Body ($body | ConvertTo-Json);
        $Global:authString = $response
    }
    catch
    {
        handleError $_;
    }

    Write-Host([string]::Format("Authorization header string generated: {0}", $Global:authString));
    Write-Host("Setup Complete...");
    exitMethod($methodName);
    return $true;
}

<#
    * Logging for method entry.
    *
    * @param methodName
    *       Name of the method to log
#>
function enterMethod($methodName)
{
    Write-Host([string]::Format("Entering {0}...", $methodName));
}

<#
    * Logging for method exit.
    *
    * @param methodName
    *       Name of the method to log.
#>
function exitMethod($methodName)
{
    Write-Host([string]::Format("Exiting {0}...", $methodName));
}

<#
    * Handle any errors that occur when performing the REST call.
    *
    * @param error
    *       The group create context.
#>
function handleError($error)
{
    Write-Error ("The command cound not be completed.  `r`nReason: " + $error.exception.message + " `r`nDetails: "+ $error.errordetails);
    exit(1);
}

function runCertificateAction()
{
    $methodName = "runCertificateAction()";
    enterMethod($methodName);

    $request = New-Object -TypeName System.Text.StringBuilder;
    $response = $null;

    # Create and set the headers for the REST request
    $headers = @{"Authorization" = $Global:authString};

    $request.Append($global:uemBaseUrl);
    $request.Append("/pki/certificates/");
    $request.Append($certificateGuid);

    # Invoke RESTMethod to call API to obtain user embedded certificate in the system
    try
    {
        if($Global:isGet)
        {
            $response = Invoke-RestMethod -Method GET -Headers $headers -Uri $request.ToString();
        }
        else
        {
            $response = Invoke-RestMethod -Method DELETE -Headers $headers -Uri $request.ToString();
        }
    }
    catch
    {
         handleError $_;
    }

    exitMethod($methodName);
    return $response;
}

function main()
{
    $methodName = "main()";
    $response = $null;
    enterMethod($methodName);

    try
    {
        if (setup)
        {
            $response = runCertificateAction;
        }
    }
    catch
    {
        handleError $_;
    }

    if($Global:isGet)
    {
       Write-Host($response | Select-Object -Last 1);
    }
    else
    {
        Write-Host([string]::Format("Certificate with guid {0} was deleted succesfully!", $certificateGuid));
    }

    exitMethod($methodName);
    Write-Host("Complete!");
    exit(0);
}

# Call main function.
main;